import urllib
import urlparse
import requests
from textocat.exceptions import raise_by_http_code

BASE_URL = 'http://api.textocat.com/'


class ApiStatus(object):
    """

    """

    def __init__(self, status_code, message):
        """
        """
        self.message = message
        self.status_code = status_code

    @staticmethod
    def from_json(json):
        return ApiStatus(json.get('statusCode'), json.get('message'))


class BatchStatus(object):
    """

    """
    IN_PROGRESS = 'IN_PROGRESS'
    FINISHED = 'FINISHED'

    def __init__(self, batch_id, status):
        """
        """
        self.batch_id = batch_id
        self.status = status

    @staticmethod
    def from_json(json):
        """
        """
        return BatchStatus(json.get('batchId'), json.get('status'))


class Batch(object):
    """

    """

    def __init__(self, batch_ids, documents):
        """
        """
        self.batch_ids = batch_ids
        self.documents = documents

    @staticmethod
    def from_json(json):
        """
        """
        return Batch(json.get('batchIds'),
                     [AnnotatedDocument.from_json(doc) for doc in json.get('documents')])


class Document(object):
    """

    """

    def __init__(self, text, tag=''):
        """
        """
        self.text = text
        self.tag = tag

    def to_dict(self):
        """
        """
        return {'text': self.text, 'tag': self.tag}

    @staticmethod
    def from_json(json):
        """
        """
        return Document(json.get('text'), json.get('tag'))


class AnnotatedDocument(object):
    """

    """
    SUCCESS = 'SUCCESS'
    INPUT_ERROR = 'INPUT_ERROR'
    SERVICE_ERROR = 'SERVICE_ERROR'

    def __init__(self, status, entities, tag=''):
        """
        """
        self.status = status
        self.entities = entities
        self.tag = tag


    @staticmethod
    def from_json(json):
        """
        """
        return AnnotatedDocument(json.get('status'),
                                 [Entity.from_json(e) for e in json.get('entities')],
                                 tag=json.get('tag', ''))


class Entity(object):
    """

    """
    PERSON = 'PERSON'
    ORGANIZATION = 'ORGANIZATION'
    GPE = 'GPE'
    LOCATION = 'LOCATION'
    TIME = 'TIME'
    MONEY = 'MONEY'

    def __init__(self, span, begin_offset, end_offset, category):
        """
        """
        self.span = span
        self.begin_offset = begin_offset
        self.end_offset = end_offset
        self.category = category

    @staticmethod
    def from_json(json):
        """
        """
        return Entity(json.get('span'),
                      json.get('beginOffset'),
                      json.get('endOffset'),
                      json.get('category'))


class SearchResult(object):
    """
    """

    def __init__(self, search_query, documents):
        """
        """
        self.search_query = search_query
        self.documents = documents

    @staticmethod
    def from_json(json):
        """
        """
        return SearchResult(json.get('searchQuery'),
                            [AnnotatedDocument.from_json(doc) for doc in json.get('documents')])


class TextocatApi(object):
    """
    TextocatApi
    """

    def __init__(self, auth_token):
        """
        :param auth_token:
        :return:
        """
        self.auth_token = auth_token

    def _request_url(self, endpoint):
        """

        :param endpoint:
        :return:
        """
        return '?'.join([urlparse.urljoin(BASE_URL, endpoint),
                         urllib.urlencode({'auth_token': self.auth_token})])

    def entity_queue(self, documents):
        """

        :param documents:
        :return:
        """
        r = requests.post(self._request_url('entity/queue'),
                          json=[x.to_dict() for x in documents])
        if r.status_code == requests.codes.accepted:
            json = r.json()
            return BatchStatus(json.get('batchId'), json.get('status'))

        raise_by_http_code(r.status_code)

    def entity_request(self, batch_id):
        """

        :param batch_id:
        :return:
        """
        r = requests.get(self._request_url('entity/request'),
                         params={'batchId': batch_id})
        if r.status_code == requests.codes.ok:
            json = r.json()
            return BatchStatus(json.get('batchId'), json.get('status'))

        raise_by_http_code(r.status_code)

    def entity_retrieve(self, batch_ids):
        """
        :param batch_ids:
        :return:
        """
        r = requests.get(self._request_url('entity/retrieve'),
                         params={'batchId': batch_ids})
        if r.status_code == requests.codes.ok:
            return Batch.from_json(r.json())

        raise_by_http_code(r.status_code)

    def entity_search(self, query):
        """
        :param query:
        :return:
        """
        r = requests.get(self._request_url('entity/search'),
                         params={'search_query': query})
        if r.status_code == requests.codes.ok:
            return SearchResult.from_json(r.json())

        raise_by_http_code(r.status_code)

    def status(self):
        """
        :return:
        """
        r = requests.get(self._request_url('status'))

        if r.status_code == requests.codes.ok:
            return ApiStatus.from_json(r.json())

        raise_by_http_code(r.status_code)





class TextocatApiException(Exception):
    def __init__(self, message, http_code):
        super(TextocatApiException, self).__init__(message)
        self.http_code = http_code


class WrongFormatException(TextocatApiException):
    def __init__(self):
        super(WrongFormatException, self).__init__('Wrong format.', 400)


class UnauthorizedException(TextocatApiException):
    def __init__(self):
        super(UnauthorizedException, self).__init__('User is not authorized. Pass correct auth token.', 401)


class MonthlyDataLimitException(TextocatApiException):
    def __init__(self):
        super(MonthlyDataLimitException, self).__init__('Monthly limit of processed data exhausted.', 402)


class PermissionException(TextocatApiException):
    def __init__(self):
        super(PermissionException, self).__init__('User don\'t have enough permissions.', 403)


class CollectionNotFountException(TextocatApiException):
    def __init__(self):
        super(CollectionNotFountException, self).__init__('Collection not found.', 404)


class UnsupportedMethodException(TextocatApiException):
    def __init__(self):
        super(UnsupportedMethodException, self).__init__('Unsupported method for this type of operation.', 405)


class UnsupportedInputException(TextocatApiException):
    def __init__(self):
        super(UnsupportedInputException, self).__init__('Unsupported input', 406)


class CollectionInProcessException(TextocatApiException):
    def __init__(self):
        super(CollectionInProcessException, self).__init__('Collection in progress.', 406)


class InputDataLimitException(TextocatApiException):
    def __init__(self):
        super(InputDataLimitException, self).__init__('Input data limit exceeded', 413)


class UnsupportedMimeTypeException(TextocatApiException):
    def __init__(self):
        super(UnsupportedMimeTypeException, self).__init__('This MIME type is not supported.', 415)


class RequestsLimitException(TextocatApiException):
    def __init__(self):
        super(RequestsLimitException, self).__init__('Requests limit exceeded.', 416)


class ConcurrentConnectionsLimitException(TextocatApiException):
    def __init__(self):
        super(ConcurrentConnectionsLimitException, self).__init__(
            'You have exceeded the number of concurrent connections.', 429)


class ServiceUnavailable(TextocatApiException):
    def __init__(self):
        super(ServiceUnavailable, self).__init__('Service unavailable.', 503)


class InternalErrorException(TextocatApiException):
    def __init__(self):
        super(InternalErrorException, self).__init__('Internal error.', 500)


class UnexpectedHttpCodeException(Exception):
    def __init__(self, http_code):
        super(UnexpectedHttpCodeException, self).__init__(str.format('Unexpected http code {0}', http_code))


def raise_by_http_code(http_code):
    excepts = {
        400: WrongFormatException,
        401: UnauthorizedException,
        402: MonthlyDataLimitException,
        403: PermissionException,
        404: CollectionNotFountException,
        405: UnsupportedMethodException,
        406: UnsupportedInputException,
        413: InputDataLimitException,
        415: UnsupportedMimeTypeException,
        416: RequestsLimitException,
        429: ConcurrentConnectionsLimitException,
        500: InternalErrorException,
        503: ServiceUnavailable,
    }

    if http_code in excepts:
        raise excepts[http_code]()

    raise UnexpectedHttpCodeException(http_code)


from setuptools import setup, find_packages

setup(
    name='textocat',
    version='0.1.0',
    description='Unofficial Textocat Python SDK',
    url='http://github.com/radik/textocat-python-sdk',
    author='Radik Fattakhov',
    author_email='radikft@gmail.com',
    license='Apache License, Version 2.0',
    packages=find_packages(exclude=['examples', 'docs', 'tests*']),
    install_requires=['requests>=2.7.0'],
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering :: Information Analysis'
    ],
    extras_require={
        'test': ['httpretty'],
    },
)